/* Arduino ultrasonic sensor hack for robot mower

---For Ambrogio L50 wiring------------------------------------------------------

Robot motor board pinout:
11---VCC    
12---GND    
15---motor current sense    (we use this pin to indicate obstacles)
19---motor direction output (we use this pin to detect moving direction)
18---motor PWM output       (we use this pin to detect speed)

             ^ FORWARD ^
               --------
              |        |
left motor   I|  TOP   |I    right motor
              |        |
              |  L50   |
charge pin --I|        |I-- charge pin
              | BOTTOM |
               --------
--------------------------------------------------------------------------------

*/

//#include <NewPing.h>


// ---- Arduino pins -------------------------------------------------
// top ultrasonic sensor  
#define TRIGGER_TOP_PIN  9
#define ECHO_TOP_PIN     10
// bottom ultrasonic sensor 2
#define TRIGGER_BOTTOM_PIN  11 
#define ECHO_BOTTOM_PIN     12 
// left motor pins
#define MOTOR_L_S_PIN     A0
#define MOTOR_L_DIR_PIN   4
#define MOTOR_L_PWM_PIN   A1
// right motor pins
#define MOTOR_R_S_PIN     A2
#define MOTOR_R_DIR_PIN   5
#define MOTOR_R_PWM_PIN   A3

// other pins
#define LED_PIN      13

// -------------------------------------------------------------
#define MAX_DISTANCE 200 // Maximum distance we want to ping for (in centimeters). Maximum sensor distance is rated at 400-500cm.
#define NO_ECHO 0
#define MAX_ECHO_TIME 5000
#define PING_MEDIAN_DELAY 29



enum DIR
{
  FORWARD,
  BACKWARD
};


//NewPing sonar(TRIGGER_PIN, ECHO_PIN, MAX_DISTANCE); // NewPing setup of pins and maximum distance.

void setup() {
  pinMode(MOTOR_L_S_PIN, INPUT);   // set bumper pin to tri-state (high impedance)
  pinMode(MOTOR_L_DIR_PIN, INPUT);   // set bumper pin to tri-state (high impedance)
  pinMode(MOTOR_L_PWM_PIN, INPUT);   // set bumper pin to tri-state (high impedance)
  
  pinMode(MOTOR_R_S_PIN, INPUT);   // set bumper pin to tri-state (high impedance)
  pinMode(MOTOR_R_DIR_PIN, INPUT);   // set bumper pin to tri-state (high impedance)
  pinMode(MOTOR_R_PWM_PIN, INPUT);   // set bumper pin to tri-state (high impedance)
    
  digitalWrite(MOTOR_L_S_PIN, LOW); // disable pin pull-up
  digitalWrite(MOTOR_L_DIR_PIN, LOW); // disable pin pull-up
  digitalWrite(MOTOR_L_PWM_PIN, LOW); // disable pin pull-up

  digitalWrite(MOTOR_R_S_PIN, LOW); // disable pin pull-up
  digitalWrite(MOTOR_R_DIR_PIN, LOW); // disable pin pull-up
  digitalWrite(MOTOR_R_PWM_PIN, LOW); // disable pin pull-up
  
  pinMode(TRIGGER_TOP_PIN, OUTPUT);
  pinMode(ECHO_TOP_PIN, INPUT);
  pinMode(TRIGGER_BOTTOM_PIN, OUTPUT);
  pinMode(ECHO_BOTTOM_PIN, INPUT);
  pinMode(LED_PIN, OUTPUT);  
  Serial.begin(115200); // Open serial monitor at 115200 baud to see ping results.  
}

unsigned int ping(uint8_t triggerpin, uint8_t echopin){
  unsigned int uS;  
  digitalWrite(triggerpin, LOW); 
  delayMicroseconds(2); 
  digitalWrite(triggerpin, HIGH);
  delayMicroseconds(10); 
  digitalWrite(triggerpin, LOW);
  uS = pulseIn(echopin, HIGH, MAX_ECHO_TIME + 1000);
  if (uS > MAX_ECHO_TIME) uS = NO_ECHO;
  return uS;
}


unsigned int ping_median(uint8_t triggerpin, uint8_t echopin, uint8_t it) {
  unsigned int uS[it], last;
  uint8_t j, i = 0;
  uS[0] = NO_ECHO;
  while (i < it) {
    last = ping(triggerpin, echopin);           // Send ping.
    if (last == NO_ECHO) {   // Ping out of range.
	it--;                // Skip, don't include as part of median.
	last = MAX_ECHO_TIME; // Adjust "last" variable so delay is correct length.
    } else {                       // Ping in range, include as part of median.
	if (i > 0) {               // Don't start sort till second ping.
		for (j = i; j > 0 && uS[j - 1] < last; j--) // Insertion sort loop.
		  uS[j] = uS[j - 1]; // Shift ping array to correct position for sort insertion.
	 } else j = 0;              // First ping is starting point for sort.
	 uS[j] = last;              // Add last ping to array in sorted position.
         i++;                       // Move to next ping.
     }
     if (i < it) delay(PING_MEDIAN_DELAY - (last >> 10)); // Millisecond delay between pings.
   }
   return (uS[it >> 1]); // Return the ping distance median.
}


static unsigned long lastBumperTime = 0;
static unsigned long nextInfo = millis() + 1000;
static long dist = 0;     
static enum DIR dir = FORWARD;
static int motorLeftSpeed = 0;
static int motorRightSpeed = 0;
static int motorLeftSense = 0;
static int motorRightSense = 0;
  
void loop() {
  long uS; 
  
  delay(50);   
  
  motorLeftSpeed = analogRead(MOTOR_L_PWM_PIN);    
  motorRightSpeed = analogRead(MOTOR_R_PWM_PIN);    
  //motorLeftSense = analogRead(MOTOR_L_S_PIN);    
  //motorRightSense = analogRead(MOTOR_R_S_PIN);    
  
  if (digitalRead(MOTOR_L_DIR_PIN) == 1) dir = FORWARD;
    else dir = BACKWARD;

  if ((millis() > lastBumperTime + 3000)){          
    // last bumper timeout expired
    if ((motorLeftSpeed > 100) || (motorRightSpeed > 100))
    {
      // robot is moving
      if (dir == FORWARD){
        // moving forward
        uS = ping_median(TRIGGER_TOP_PIN, ECHO_TOP_PIN, 5);             
        dist = uS/57;  
        if ((dist > 0) && (dist  < 30)){        
          digitalWrite(LED_PIN, HIGH);            
          // trigger short high pulse      
          pinMode(MOTOR_L_S_PIN, OUTPUT);   
          delay(100);
          digitalWrite(MOTOR_L_S_PIN, HIGH);      
          delay(100);
          digitalWrite(MOTOR_L_S_PIN, LOW);          
          pinMode(MOTOR_L_S_PIN, INPUT);   
          lastBumperTime = millis();        
        }      
      } else {
        // moving backward      
        uS = ping_median(TRIGGER_BOTTOM_PIN, ECHO_BOTTOM_PIN, 5);             
        dist = uS/57;        
        if ((dist > 0) && (dist < 30)){        
          digitalWrite(LED_PIN, HIGH);            
          // trigger short high pulse      
          pinMode(MOTOR_L_S_PIN, OUTPUT);             
          delay(100);
          digitalWrite(MOTOR_L_S_PIN, HIGH);
          delay(100);          
          digitalWrite(MOTOR_L_S_PIN, LOW);          
          pinMode(MOTOR_L_S_PIN, INPUT);   
          lastBumperTime = millis();        
        }
      }
    }    
  }
 
  if ((millis() > lastBumperTime + 1000)) {
    digitalWrite(LED_PIN, LOW);    
  }
  
  if (millis() > nextInfo){    
    nextInfo = millis() + 200;
    Serial.print("dir ");
    if (dir == FORWARD) Serial.print("FORW");
      else Serial.print("BACK");
    Serial.print(" lspeed ");
    Serial.print(motorLeftSpeed);
    Serial.print(" rspeed ");    
    Serial.print(motorRightSpeed);    
    Serial.print(" lsense ");
    Serial.print(motorLeftSense);
    Serial.print(" rsense ");    
    Serial.print(motorRightSense);    
    Serial.print(" uS ");
    Serial.print(uS); 
    Serial.print(" dist ");
    Serial.print(dist); 
    /*Serial.print(" freq ");
    Serial.print(freq); 
    Serial.print(" state ");
    switch (state)
    {
       case BUMPER1:  Serial.print("B1"); break;
       case BUMPER2:  Serial.print("B2"); break;
       case OFF:  Serial.print("OFF"); break;       
    }*/
    Serial.println(""); 
  }
  
        
}


